/*
 * grader-io - compiler.h
 *
 * Copyright (C) 2010, 2016  Krzysztof Mazur <krzysiek@podlesie.net>
 * based on Linux code under GPLv2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifndef __COMPILER_H
#define __COMPILER_H

#if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 96)
# define __builtin_expect(x, expected_value) (x)
#endif

#define likely(x)	__builtin_expect((x), 1)
#define unlikely(x)	__builtin_expect((x), 0)

#if __GNUC__
# define barrier()	__asm__ __volatile__ ("" : : : "memory")
#else
# define barrier()	do { } while (0)
#endif

#if (__GNUC > 4) || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1)
#define mb() __sync_synchronize()
#define rmb() mb()
#define wmb() mb()
#else
#define mb()				barrier()
#define rmb()				mb()
#define wmb()				mb()
#endif

#define __iomem

#if __GNUC__ >= 4
#define __compiler_offsetof(a, b) __builtin_offsetof(a, b)
#endif

#undef offsetof
#ifdef __compiler_offsetof
#define offsetof(TYPE, MEMBER) __compiler_offsetof(TYPE, MEMBER)
#else
#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
#endif

/*
 * container_of - cast a member of a structure out to the containing structure
 * @ptr:	the pointer to the member.
 * @type:	the type of the container struct this is embedded in.
 * @member:	the name of the member within the struct.
 */
#define container_of(ptr, type, member) ({			\
	typeof( ((type *)0)->member ) *__mptr = (ptr);	\
	(type *)(void *)( (char *)__mptr - offsetof(type,member) );})

#define get_unaligned(ptr)						\
	({								\
		const struct __str {					\
			typeof(*(ptr)) __v;				\
	 	} __attribute__((packed)) *__p = (const struct __str *) \
	 		(ptr);						\
		__p->__v;						\
	})

#define put_unaligned(val, ptr)						\
	do {								\
		struct __str {						\
			typeof(*(ptr)) __v;				\
		} __attribute__((packed)) *__p = (struct __str *) (ptr);\
		__p->__v = (val);					\
	} while(0)

#define __iomem

#define smp_mb() barrier()
#define smp_wmb() barrier()
#define smp_rmb() barrier()
#define smp_read_barrier_depends() barrier()

/*
 * Prevent the compiler from merging or refetching accesses.  The compiler
 * is also forbidden from reordering successive instances of ACCESS_ONCE(),
 * but only when the compiler is aware of some particular ordering.  One way
 * to make the compiler aware of ordering is to put the two invocations of
 * ACCESS_ONCE() in different C statements.
 *
 * This macro does absolutely -nothing- to prevent the CPU from reordering,
 * merging, or refetching absolutely anything at any time.
 */
#define ACCESS_ONCE(x) (*(volatile typeof(x) *)&(x))

#define __user

#endif
