/*
 * grader-io - serial.c
 *
 * NXP LPC111x serial driver
 *
 * Copyright (C) 2016  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <kernel.h>
#include <asm_io.h>
#include <lpc176x.h>
#include <serial_reg.h>
#include <uart-nxp.h>
#include <tty.h>

#define NR_UARTS		4

static struct uart uart[NR_UARTS];

struct uart_config {
	unsigned long base_addr;
	unsigned int irq;
};

static struct uart_config uart_config[] = {
	{ 0x4000c000, 5 },
	{ 0x40010000, 6 },
	{ 0x40098000, 7 },
	{ 0x4009c000, 8 },
};

unsigned long serial_irq_time;
unsigned long gettime(void);

void uart0_irq_handler(void) __attribute__((interrupt("IRQ")))
	__attribute__((hot));

void uart0_irq_handler(void)
{
	uart_irq_handler(&uart[0]);
}

void uart1_irq_handler(void) __attribute__((interrupt("IRQ")))
	__attribute__((hot));

void uart1_irq_handler(void)
{
	uart_irq_handler(&uart[1]);
}

void uart2_irq_handler(void) __attribute__((interrupt("IRQ")))
	__attribute__((hot));

void uart2_irq_handler(void)
{
	uart_irq_handler(&uart[2]);
}

void uart3_irq_handler(void) __attribute__((interrupt("IRQ")))
	__attribute__((hot));

void uart3_irq_handler(void)
{
	uart_irq_handler(&uart[3]);
}

void serial_poll(void)
{
}

int putchar(int c)
{
	uart[3].tty.ops->write(&uart[3].tty, (unsigned char *) &c, 1);
//	uart[0].tty.ops->write(&uart[0].tty, (unsigned char *) &c, 1);
	return 1;
}

static void dummy_receive_buf(struct tty_struct *tty, const unsigned char *cp,
		char *fp, int count)
{
}

static const struct tty_ldisc_ops dummy_ldisc_ops = {
	.receive_buf		= dummy_receive_buf,
};

struct tty_struct *serial_init(unsigned int port)
{
	struct uart *p = &uart[port];

	p->tty.ldisc_ops = &dummy_ldisc_ops;

//	uart_init(p, uart_config[port].base_addr, uart_config[port].irq, 2);
	uart_init(p, uart_config[port].base_addr, uart_config[port].irq, 4);
//	uart_init(p, uart_config[port].base_addr, uart_config[port].irq, 2);
	return &p->tty;
}
